/*
   CRT - Guest - SM  Shader
   
   Copyright (C) 2019 guest(r) - guest.r@gmail.com
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
   
*/


#define brightboost1 1.30     // adjust brightness - dark pixels (1.0 to 2.0)
#define brightboost2 1.15     // adjust brightness - bright pixels (1.0 to 2.0)
#define stype        2.00     // scanline type (0.0, 1.0, 2.0)
#define scanline1    8.00     // scanline shape, center (3.0 to 12.0)
#define scanline2    8.00     // scanline shape, edges (6.0 to 15.0)
#define beam_min     1.50     // dark area beam min - narrow (0.5 to 2.0)
#define beam_max     1.10     // bright area beam max - wide (0.5 to 2.0)
#define s_beam       0.90     // overgrown bright beam (0.0 to 1.0)
#define saturation1  1.75     // scanline saturation (0.0 to 4.0)
#define h_sharp      1.50     // pixel sharpness (1.0 to 5.0)
#define gamma_out    2.30     // gamma out


// The name of this effect
string name : NAME = "CrtGuest";

float2 ps1                      : TEXELSIZE;
float2 ir                       : sourcescale;

float4x4 World                  : WORLD;
float4x4 View                   : VIEW;
float4x4 Projection             : PROJECTION;
float4x4 Worldview              : WORLDVIEW;               // world * view
float4x4 ViewProjection         : VIEWPROJECTION;          // view * projection
float4x4 WorldViewProjection    : WORLDVIEWPROJECTION;     // world * view * projection

string combineTechique          : COMBINETECHNIQUE = "CrtGuest";

texture SourceTexture	        : SOURCETEXTURE;
texture WorkingTexture          : WORKINGTEXTURE;

sampler	decal = sampler_state
{
	Texture	  = (SourceTexture);
	MinFilter = LINEAR;
	MagFilter = LINEAR;
};


struct out_vertex {
	float4 position : POSITION;
	float4 color    : COLOR;
	float2 t0       : TEXCOORD0;
}; 
 

out_vertex  VS_VERTEX(float3 position : POSITION, float2 texCoord : TEXCOORD0 )
{ 
	out_vertex OUT = (out_vertex)0;

	OUT.position = mul(float4(position,1.0),WorldViewProjection);
	OUT.t0 = texCoord;
	return OUT;  
}


float st(float x)
{
	return exp2(-10.0*x*x);
}  

float st1(float x, float scan)
{
	return exp2(-scan*x*x);
}  

float sw1(float x, float3 color, float scan)
{	
	float mx = max(max(color.r,color.g),color.b);
	float ex = lerp((2.75 - 1.75*stype)*beam_min, beam_max, mx);
	ex = lerp(beam_max, ex, pow(x, mx + 0.25))*x;
	return exp2(-scan*ex*ex);
}

float sw2(float x, float3 color)
{	
	float mx = max(max(color.r,color.g),color.b);
	float ex = lerp(2.0*beam_min, beam_max, mx);
	float m = 0.5*ex;
	x = x*ex; float xx = x*x;
	xx = lerp(xx, x*xx, m);
	return exp2(-10.0*xx);
}

float4 PS_FRAGMENT (in out_vertex VAR) : COLOR
{	
	float2 tmp = (ir.x < 1.0) ? float2(1.0,1.0) : ir;
	float2 ps = ps1*tmp;
	
	float2 OGL2Pos = VAR.t0 / ps - float2(0.5,0.5);
	float2 fp = frac(OGL2Pos);
	float2 dx = float2(ps.x,0.0);
	float2 dy = float2(0.0, ps.y);
	
	float2 pC4 = floor(OGL2Pos) * ps + 0.5*ps;	
	
	// Reading the texels
	float3 ul = tex2D(decal, pC4     ).xyz; ul*=ul;
	float3 ur = tex2D(decal, pC4 + dx).xyz; ur*=ur;
	float3 dl = tex2D(decal, pC4 + dy).xyz; dl*=dl;
	float3 dr = tex2D(decal, pC4 + ps).xyz; dr*=dr;
	
	float lx = fp.x;        lx = pow(lx, h_sharp);
	float rx = 1.0 - fp.x;  rx = pow(rx, h_sharp);
	
	float w = 1.0/(lx+rx);
	float f1 = fp.y;
	float f2 = 1.0 - fp.y;
	float f3 = frac(VAR.t0.y / ps.y); f3 = abs(f3-0.5);
	
	float3 color;
	float t1 = st(f1);
	float t2 = st(f2);
	float wt = 1.0/(t1+t2);
	
// calculating scanlines

	float3 cl = (ul*t1 + dl*t2)*wt;
	float3 cr = (ur*t1 + dr*t2)*wt;	
	
	float3 ref_ul = lerp(cl, ul, s_beam);
	float3 ref_ur = lerp(cr, ur, s_beam);
	float3 ref_dl = lerp(cl, dl, s_beam);
	float3 ref_dr = lerp(cr, dr, s_beam);	
	
	float scan1 = lerp(scanline1, scanline2, f1);
	float scan2 = lerp(scanline1, scanline2, f2);
	float scan0 = lerp(scanline1, scanline2, f3);
	f3 = st1(f3,scan0);
	f3 = f3*f3*(3.0-2.0*f3);
	
	float w1, w2, w3, w4 = 0.0;

	if (stype < 2.0)
	{
		w1 = sw1(f1, ref_ul, scan1);
		w2 = sw1(f2, ref_dl, scan2);
		w3 = sw1(f1, ref_ur, scan1);
		w4 = sw1(f2, ref_dr, scan2);
	}
	else
	{
		w1 = sw2(f1, ref_ul);
		w2 = sw2(f2, ref_dl);
		w3 = sw2(f1, ref_ur);
		w4 = sw2(f2, ref_dr);	
	}

	float3 colorl = w1*ul + w2*dl;
	float3 colorr = w3*ur + w4*dr;
	color = w*(colorr*lx + colorl*rx);
	color = min(color,1.0);	
	
	float3 ctemp = w*(cr*lx + cl*rx);

	float mx1 = max(max(color.r,color.g),color.b);
	float sp = (stype == 1.0) ? (0.5*saturation1) : saturation1;
	float3 saturated_color = max((1.0+sp)*color - sp*mx1, 0.0);
	color = lerp(saturated_color, color, f3);
	
	color*=lerp(brightboost1, brightboost2, max(max(ctemp.r,ctemp.g),ctemp.b));
	color = min(color,1.0);	
	
	float3 color1 = pow(color, float3(1.0,1.0,1.0)/2.1);

	if (!(stype == 1.0))
	{
		float3 color2 = pow(color, float3(1.0,1.0,1.0)/gamma_out);			
		mx1 = max(max(color1.r,color1.g),color1.b) + 1e-12;	
		float mx2 = max(max(color2.r,color2.g),color2.b);
		color1*=mx2/mx1;		
	}
	
    return float4(color1, 1.0);	
}


//
// Technique
//

technique CrtGuest
{
    pass P0
    {
        // shaders		
        VertexShader = compile vs_3_0 VS_VERTEX();
        PixelShader  = compile ps_3_0 PS_FRAGMENT(); 
    }  
}

